﻿using FreeRedis;
using NetTaste;
using Newtonsoft.Json;
using NPOI.SS.Formula.Functions;
using NPOI.SS.Formula.PTG;
using ServiceCenter.Extensions;
using ServiceCenter.Redis;
using ServiceCenter.SqlSugars;
using ServiceCenter.WebApi;
using SqlSugar;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.SRM;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.WebApi.Models.WCS.Request;
using WCS.WorkEngineering.Worlds;

namespace WCS.WorkEngineering.Extensions
{
    /// <summary>
    /// 堆垛机扩展
    /// </summary>
    public class SRM : Device<ISRM520, ISRM521, ISRM542>
    {
        public Device<IStation521, IStation523>[] ConvInArr { get; private set; }
        public Device<IStation521, IStation523>[] ConvOutArr { get; private set; }

        public int Tunnel { get; private set; }
        public int Floor { get; private set; }

        short _inout;
        bool InOut
        {
            get
            {
                _inout++;
                return _inout % 2 == 0;
            }
        }

        public SRM(Device device, World world) : base(device, world)
        {
            Tunnel = int.Parse(device.GetFlag("Tunnel"));
            Floor = int.Parse(device.GetFlag("Floor"));

            //var st = Device.All.Where(v => v.HasProtocol<IStation521>() && v.HasFlag("Inbound"))
            //   .Where(v => v.GetFlag("Floor") == Floor.ToString())
            //   .Where(v => v.GetFlag("Tunnel") == Tunnel.ToString())
            //   .Select(v => new Device<IStation521, IStation523>(v, world)).FirstOrDefault();
            //var st2 = Device.All.Where(v => v.Code == (int.Parse(st.Entity.Code) - 1).ToString())
            //    .Select(v => new Device<IStation521, IStation523>(v, world)).FirstOrDefault();

            var st = Device.All.Where(v => v.HasProtocol<IStation521>() && v.HasFlag(DeviceFlags.入库 | DeviceFlags.巷道口))
                .Where(v => v.GetFlag("Floor") == Floor.ToString())
               .Where(v => v.GetFlag("Tunnel") == Tunnel.ToString())
               .Select(v => new Device<IStation521, IStation523>(v, world)).FirstOrDefault();

            var st2 = Device.All.Where(v => v.Code == (int.Parse(st.Entity.Code) + 1).ToString())
                .Select(v => new Device<IStation521, IStation523>(v, world)).FirstOrDefault();

            ConvInArr = new Device<IStation521, IStation523>[] { st2, st };

            ConvOutArr= Device.All.Where(v => v.HasFlag("位置", "堆垛机放货工位") && v.HasFlag("Floor", this.Floor.ToString())&&v.HasFlag("Tunnel", Tunnel.ToString()))
                .Select(v => new Device<IStation521, IStation523>(v, world)).ToArray();
             
        }


        public void Do()
        {
            var isOut = false;
            SqlSugarHelper.Do(db =>
            {
                ////获取所有堆垛机中的码垛出库
                //var tasks = db.Default.Queryable<WCS_TaskInfo>().With(SqlWith.NoLock).Where(x => x.BusType == "装箱码垛" && x.Status < WCS.Entity.TaskStatus.堆垛机执行).ToList().GroupBy(x => x.Device).Select(x => new
                //{
                //    x.Key,
                //    total = x.Count()
                //}).OrderBy(x => x.total).ToArray();
                //var length = tasks.Length;
                //double median = 0;
                //if (length % 2 == 0)
                //{
                //    // 数列长度为偶数，取中间两个数的平均值
                //    int middleIndex1 = length / 2 - 1;
                //    int middleIndex2 = length / 2;
                //    median = (tasks[middleIndex1].total + tasks[middleIndex2].total) / 2.0;
                //}
                //else
                //{
                //    // 数列长度为奇数，取中间值
                //    int middleIndex = length / 2;
                //    median = tasks[middleIndex].total;
                //}
                //median = median * 1.5; //按照中位数1.5倍判断，如果多余这个数。此堆垛机则不执行入库任务
                //tasks = tasks.Where(x => x.total > median).ToArray();
                //isOut = tasks.Any(x => x.Key == Entity.Code);
                var tasks = db.Default.Queryable<WCS_TaskInfo>().With(SqlWith.NoLock).Where(x => x.BusType == "装箱码垛" && x.Status < WCS.Entity.TaskStatus.堆垛机执行).ToList().GroupBy(x => x.PalletizingID)
                .Where(x => x.Count() < 10).SelectMany(x => x).Select(x => x.Device).ToList();
                
                isOut = tasks.Any(x => x == Entity.Code) ;
            });
            if (InOut && !isOut)
            {//入库
                try
                {
                    World.Log("开始获取入库任务");
                    if (!ConvInArr[0].Data2.Status.HasFlag(StationStatus.高位))
                        return;
                    if (ConvInArr[0].Data.TaskNumber == 0)
                    {
                        throw new Exception("入库顶升在高位，但任务号为0");
                    }
                    var tasks = ConvInArr.Select(v => v.Data.TaskNumber).Where(v => v != 0).ToArray();
                    var res = WMS.入库分配(tasks, Tunnel, Floor);
                    if (!res.Success)
                    {
                        MainWorld.Redis.Set($"Sx:WCSErrorInfo:{Tunnel}巷道{Floor}层", JsonConvert.SerializeObject(new RedisError() { Equip = string.Join(",", tasks), Con = "堆垛机" + Entity.Code + "，任务号：" + string.Join(",", tasks) + res.ResMsg, Time = DateTime.Now }));
                        return;
                    }

                    if (res.ResData.GroupBy(v => v.Fork).Where(v => v.Count() > 1).Any())
                    {
                        MainWorld.Redis.Set($"Sx:WCSErrorInfo:{Tunnel}巷道{Floor}层", JsonConvert.SerializeObject(new RedisError() { Equip = string.Join(",", tasks), Con = "堆垛机" + Entity.Code + "，任务号：" + string.Join(",", tasks) + "同一货叉分配了多个任务", Time = DateTime.Now }));
                        throw new Exception("同一货叉分配了多个任务");
                    }

                    SqlSugarHelper.Do(db =>
                    {
                        foreach (var loc in res.ResData)
                        {
                            db.Default.Updateable<WCS_TaskInfo>()
                            .SetColumns(v => new WCS_TaskInfo { AddrTo = loc.CellNo, Device = Entity.Code, Fork = loc.Fork, Status = WCS.Entity.TaskStatus.堆垛机执行, EditTime = SqlFunc.GetDate() })
                            .Where(v => v.ID == loc.TaskNo)
                            .ExecuteCommand();
                        }
                        var info = res.ResData.Where(v => v.Fork == 1).FirstOrDefault();


                        if (info != null)
                        {
                            if (Data2.TaskID1 == info.TaskNo)
                                throw new Exception("禁止重复下发任务");
                            Data.TaskID1 = info.TaskNo;
                            Data.SRow1 = 0;
                            Data.SCol1 = 801;
                            Data.SLayer1 = 0;
                            Data.ERow1 = (short)info.Row;
                            Data.ECol1 = (short)info.Colomn;
                            Data.ELayer1 = (short)info.Layer;
                            Data.Target1 = 0;
                            Data.TaskType1 = 4;
                        }
                        else
                        {
                            Data.TaskID1 = 0;
                            Data.SRow1 = 0;
                            Data.SCol1 = 0;
                            Data.SLayer1 = 0;
                            Data.ERow1 = 0;
                            Data.ECol1 = 0;
                            Data.ELayer1 = 0;
                            Data.Target1 = 0;
                            Data.TaskType1 = 0;
                        }

                        info = res.ResData.Where(v => v.Fork == 2).FirstOrDefault();
                        if (info != null)
                        {
                            if (Data2.TaskID2 == info.TaskNo)
                                throw new Exception("禁止重复下发任务");
                            Data.TaskID2 = info.TaskNo;
                            Data.SRow2 = 0;
                            Data.SCol2 = 802;
                            Data.SLayer2 = 0;
                            Data.ERow2 = (short)info.Row;
                            Data.ECol2 = (short)info.Colomn;
                            Data.ELayer2 = (short)info.Layer;
                            Data.Target2 = 0;
                            Data.TaskType2 = 4;
                        }
                        else
                        {
                            Data.TaskID2 = 0;
                            Data.SRow2 = 0;
                            Data.SCol2 = 0;
                            Data.SLayer2 = 0;
                            Data.ERow2 = 0;
                            Data.ECol2 = 0;
                            Data.ELayer2 = 0;
                            Data.Target2 = 0;
                            Data.TaskType2 = 0;
                        }
                        Data.VoucherNo++;
                        World.Log("入库任务下发成功");
                    });
                }
                catch (InvokingException ex)
                {
                    var a = InOut;
                }
            }
            else
            {//出库

                if (ConvOutArr.Where(v => v.Data.TaskNumber > 0).Any())
                {
                    World.Log("放货工位有货");
                    return;
                }

                var devout = Device.All.Where(v => v.HasFlag("出库扫码") && v.HasFlag("Floor", this.Floor.ToString())).First();
                var code = short.Parse(devout.Code);
                World.Log("开始获取出库任务");
                var view = MainWorld.Redis.HGet<V_TaskOut>("StockTasks", Entity.Code);
                if (view == null)
                    return;
                SqlSugarHelper.Do(db =>
                {
                    //var view = db.Default.SqlQueryable<V_TaskOut>($"select * from F_GetTaskOut('{Entity.Code}')").First();
                    var taskIdArr = new int[] { view.Task1, view.Task2 };
                    var taskQty = taskIdArr.Where(v => v != 0).Count();
                    //if (view.BusType1 == "装箱码垛" || view.BusType2 == "装箱码垛")
                    //{
                    //    //var stockOperateNum = MainWorld.Redis.IncrBy($"stockOperateNum{view.AddrTo}", taskQty);
                    //    //if (stockOperateNum > 11 && !string.IsNullOrEmpty(view.AddrTo) )
                    //    //{
                    //    //    MainWorld.Redis.DecrBy($"stockOperateNum{view.AddrTo}", taskQty);
                    //    //    return;
                    //    //}
                    //}
                    //List<string> srm3list = new List<string>() { "SRM301", "SRM302", "SRM303", "SRM304", "SRM305", "SRM306", "SRM307", "SRM308" };
                    //List<string> srm2list = new List<string>() { "SRM201", "SRM202", "SRM203", "SRM204", "SRM205", "SRM206", "SRM207", "SRM208" };
                    //List<string> srm1list = new List<string>() { "SRM101", "SRM102", "SRM103", "SRM104", "SRM105", "SRM106", "SRM107", "SRM108" };
                    //if (srm1list.Contains(view.SRM) && (view.Type1 == TaskType.OutDepot.GetHashCode() || view.Type2 == TaskType.OutDepot.GetHashCode()))
                    //{
                    //    var t1 = MainWorld.Redis.Get("Floor1Torsch");
                    //    var t2 = MainWorld.Redis.Get("Floor1Preel");
                    //    var t3 = MainWorld.Redis.Get("TaskCountNo");
                    //    if (int.Parse(t1) + int.Parse(t2) >= int.Parse(t3))
                    //    {
                    //        MainWorld.Redis.Set($"Sx:WCSErrorInfo:{view.SRM}", JsonConvert.SerializeObject(new RedisError() { Equip = view.SRM, Con = view.SRM + ",一楼出库转圈任务较多，暂停下发新的出库任务", Time = DateTime.Now }));
                    //        World.Log("一楼出库转圈任务较多，暂停下发新的出库任务");
                    //        return;
                    //    }
                    //}
                    //else if (srm2list.Contains(view.SRM) && (view.Type1 == TaskType.OutDepot.GetHashCode() || view.Type2 == TaskType.OutDepot.GetHashCode()))
                    //{
                    //    var t1 = MainWorld.Redis.Get("Floor2Torsch");
                    //    var t2 = MainWorld.Redis.Get("Floor2Preel");
                    //    var t3 = MainWorld.Redis.Get("TaskCountNo");
                    //    if (int.Parse(t1) + int.Parse(t2) >= int.Parse(t3))
                    //    {
                    //        MainWorld.Redis.Set($"Sx:WCSErrorInfo:{view.SRM}", JsonConvert.SerializeObject(new RedisError() { Equip = view.SRM, Con = view.SRM + ",二楼出库转圈任务较多，暂停下发新的出库任务", Time = DateTime.Now }));
                    //        World.Log("二楼出库转圈任务较多，暂停下发新的出库任务");
                    //        return;
                    //    }
                    //}
                    //else if (srm3list.Contains(view.SRM) && (view.Type1 == TaskType.OutDepot.GetHashCode() || view.Type2 == TaskType.OutDepot.GetHashCode()))
                    //{
                    //    var t1 = MainWorld.Redis.Get("Floor3Torsch");
                    //    var t2 = MainWorld.Redis.Get("Floor3Preel");
                    //    var t3 = MainWorld.Redis.Get("TaskCountNo");
                    //    if (int.Parse(t1) + int.Parse(t2) >= int.Parse(t3))
                    //    {
                    //        MainWorld.Redis.Set($"Sx:WCSErrorInfo:{view.SRM}", JsonConvert.SerializeObject(new RedisError() { Equip = view.SRM, Con = view.SRM + ",三楼出库转圈任务较多，暂停下发新的出库任务", Time = DateTime.Now }));
                    //        World.Log("三楼出库转圈任务较多，暂停下发新的出库任务");
                    //        return;
                    //    }
                    //}
                    var q = db.Default.Updateable<WCS_TaskInfo>()
                    .SetColumns(v => new WCS_TaskInfo { Status = WCS.Entity.TaskStatus.堆垛机执行, EditTime = SqlFunc.GetDate() })
                    .Where(v => taskIdArr.Contains(v.ID));
                    var sql = q.ToSqlString();
                    var rc = q.ExecuteCommand();
                    if (rc != taskQty)
                    {
                        throw new Exception("竞争出库任务失败");
                    }

                    if (view.Task1 != 0)
                    {
                        Data.TaskID1 = view.Task1;
                        Data.SRow1 = view.Row1;
                        Data.SCol1 = view.Col1;
                        Data.SLayer1 = view.Layer1;
                        if (view.Type1 == 2)
                        {
                            Data.ERow1 = 0;
                            Data.ECol1 = 901;
                            Data.ELayer1 = 0;
                            if (view.BusType1 == "装箱码垛")
                                Data.Target1 = 1;
                            else if (view.BusType1 == "扭转检测")
                                Data.Target1 = 2;
                            else
                                Data.Target1 = 3; 
                        }
                        else if (view.Type1 == 3)
                        {
                            var sSql = @"select a.Code,a.Row,a.Col,a.Layer,a.Depth
from Base_WareCell a
inner join WCS_TaskInfo b on a.Code=b.AddrTo
where b.ID=" + view.Task1;
                            var addrTo = db.Default.SqlQueryable<LocInfo>(sSql).First();
                            Data.ERow1 = (short)addrTo.Row;
                            Data.ECol1 = (short)addrTo.Col;
                            Data.ELayer1 = (short)addrTo.Layer;
                            Data.Target1 = 0;
                        }
                        else
                            throw new Exception($"任务类型不支持");
                        Data.TaskType1 = 4;
                    }
                    else
                    {
                        Data.TaskID1 = 0;
                        Data.SRow1 = 0;
                        Data.SCol1 = 0;
                        Data.SLayer1 = 0;
                        Data.ERow1 = 0;
                        Data.ECol1 = 0;
                        Data.ELayer1 = 0;
                        Data.Target1 = 0;
                        Data.TaskType1 = 0;
                    }


                    if (view.Task2 != 0)
                    {
                        Data.TaskID2 = view.Task2;
                        Data.SRow2 = view.Row2;
                        Data.SCol2 = view.Col2;
                        Data.SLayer2 = view.Layer2;
                        if (view.Type2 == 2)
                        {
                            Data.ERow2 = 0;
                            Data.ECol2 = 902;
                            Data.ELayer2 = 0;
                            if (view.BusType2 == "装箱码垛")
                                Data.Target2 = 1;
                            else if (view.BusType2 == "扭转检测")
                                Data.Target2 = 2;
                            else
                                Data.Target2 = 3;
                        }
                        else if (view.Type2 == 3)
                        {
                            var sSql = @"select a.Code,a.Row,a.Col,a.Layer,a.Depth
from Base_WareCell a
inner join WCS_TaskInfo b on a.Code=b.AddrTo
where b.ID=" + view.Task2;
                            var addrTo = db.Default.SqlQueryable<LocInfo>(sSql).First();
                            Data.ERow2 = (short)addrTo.Row;
                            Data.ECol2 = (short)addrTo.Col;
                            Data.ELayer2 = (short)addrTo.Layer;
                            Data.Target2 = 0;
                        }
                        else
                            throw new Exception($"任务类型不支持");
                        Data.TaskType2 = 4;
                    }
                    else
                    {
                        Data.TaskID2 = 0;
                        Data.SRow2 = 0;
                        Data.SCol2 = 0;
                        Data.SLayer2 = 0;
                        Data.ERow2 = 0;
                        Data.ECol2 = 0;
                        Data.ELayer2 = 0;
                        Data.Target2 = 0;
                        Data.TaskType2 = 0;
                    }
                    Data.VoucherNo++;
                    World.Log("出库任务下发成功");
                    MainWorld.Redis.HDel("StockTasks", Entity.Code);
                });
            }
        }

        public void FinishTask(SqlSugarHelper db, int taskId, string devCode)
        {
            var task = db.Default.Queryable<WCS_TaskInfo>().Where(v => v.ID == taskId).First();
            if (task != null)
            {
                if (task.Type == TaskType.EnterDepot || task.Type == TaskType.TransferDepot)
                {
                    task.Status = WCS.Entity.TaskStatus.Finish;
                    task.EditTime = DateTime.Now;
                    task.EedTime = DateTime.Now;
                    db.Default.Updateable(task).ExecuteCommand();
                    task.AddWCS_TASK_DTL(db.Default, devCode, "任务完成");
                }
                else if (task.Type == TaskType.OutDepot)
                {
                    task.Status = WCS.Entity.TaskStatus.堆垛机完成;
                    task.EditTime = DateTime.Now;
                    db.Default.Updateable(task).ExecuteCommand();
                    task.AddWCS_TASK_DTL(db.Default, devCode, "堆垛机完成");
                }
            }
        }


    }
}
