﻿using Newtonsoft.Json;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using SqlSugar;
using System.ComponentModel;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.BCR;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.WebApi.Models.WCS.Request;

//using WCS.WorkEngineering.WebApi.Controllers;
//using WCS.WorkEngineering.WebApi.Models.WMS.Request;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  帘线扫码分配楼层
    /// </summary>
    [BelongTo(typeof(CordThreadWorld))]
    [Description("帘线扫码")]
    public class 帘线 : DeviceSystem<Device<IStation520, IStation521, IStation523, IBCR81>>
    {
        protected override bool ParallelDo => true;

        public override void Do(Device<IStation520, IStation521, IStation523, IBCR81> obj)
        {
            if (obj.Data.VoucherNo != obj.Data2.VoucherNo)
                return;
            if (obj.Data2.Request == 0)
                return;

            //设备号等于起始地址情况，是电控没有扫码条码
            if (obj.Data2.GoodsStart.ToString() == obj.Entity.Code)
            {
                MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = "帘线扫码" + obj.Entity.Code + "扫码失败", Time = DateTime.Now }));
                //throw new Exception("扫码失败");

                SqlSugarHelper.Do(db =>
                {
                    var error = new BaseErrorinfoWcs()
                    {
                        ContBarCode = obj.Entity.Code,
                        Message = $"扫码失败,自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                        Count = 1,
                        Memo = "WCS"
                    };
                    db.Default.Insertable(error).ExecuteCommand();
                });

                obj.Data.TaskNumber = obj.Data2.TaskNumber;
                obj.Data.GoodsEnd = 10;
                obj.Data.VoucherNo++;
                return;
            }

            var barcode = obj.Data4.Content.Replace("\r", "");
            if (string.IsNullOrEmpty(barcode))
            {
                MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = "帘线扫码" + obj.Entity.Code + "扫码失败", Time = DateTime.Now }));
                //throw new Exception("扫码失败");

                //SqlSugarHelper.Do(db =>
                //{
                //    var error = new BaseErrorinfoWcs()
                //    {
                //        ContBarCode = obj.Entity.Code,
                //        Message = $"扫码值是空值,自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                //        Count = 1,
                //        Memo = "WCS"
                //    };
                //    db.Default.Insertable(error).ExecuteCommand();
                //});
                //if (obj.Entity.Code == "2134")
                //{
                    throw new Exception("扫码是空值");
                //}
                //else
                //{
                //    obj.Data.TaskNumber = obj.Data2.TaskNumber;
                //    obj.Data.GoodsEnd = 10;
                //    obj.Data.VoucherNo++;
                //    return;
                //}
            
                
                //var num = MainWorld.Redis.HGet<int>($"NotScanNumber{obj.Entity.Code}", DateTime.Now.ToString("yyyyMMdd"));
                //MainWorld.Redis.HSet($"NotScanNumber{obj.Entity.Code}", DateTime.Now.ToString("yyyyMMdd"), (num + 1));
                //return;
            }
            var res = WMS.帘线扫码(obj.Entity.Code, barcode);
            World.Log($"帘线扫码{obj.Entity.Code}交互开始时间:工字轮条码{barcode} | 颜色标识 {obj.Data2.GoodsStart}" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
            var ww = "";
            if (barcode.EndsWith("R"))
            {
                ww = "R";
            }
            else
            {
                ww = "L";
            }
            if (string.IsNullOrEmpty(ww))
            {
                throw new Exception("条码值不规范，不是LR结尾；条码值：" + barcode);
            }

            string color = "没有识别出颜色";
            if (obj.Data2.GoodsStart == 1)
            {
                color = "黑灰色";
            }
            else if (obj.Data2.GoodsStart == 2)
            {
                color = "橙黄色";
            }
            else if (obj.Data2.GoodsStart == 3)
            {
                color = "蓝色";
            }

            if (res.Memo2 == "A")//A等级的工字轮才做颜色识别
            {
                if (ww == "R")
                {
                    if (obj.Data2.GoodsStart != 2) // 1黑- >L; 2黄 -> R
                    {
                        SqlSugarHelper.Do(db =>
                        {
                            var error = new BaseErrorinfoWcs()
                            {
                                ContBarCode = barcode,
                                Message = $"帘线异常排出口,颜色不匹配,识别器识别为:{color},自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                                Count = 1,
                                Memo = "WCS"
                            };
                            var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                            if (errorinfoWcs != null)
                            {
                                errorinfoWcs.Count = errorinfoWcs.Count++;
                                errorinfoWcs.AddTime = DateTime.Now;
                                db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                            }
                            else
                            {
                                db.Default.Insertable(error).ExecuteCommand();
                            }
                        });

                        World.Log($"帘线扫码{obj.Entity.Code}交互结束时间:工字轮条码{barcode} | 颜色标识 {obj.Data2.GoodsStart} " + DateTime.Now.ToString("yyyyMMddHHmmss"));
                        obj.Data.TaskNumber = obj.Data2.TaskNumber;
                        obj.Data.GoodsEnd = 10;
                        obj.Data.VoucherNo++;
                        return;
                    }
                }
                else
                {
                    if (obj.Data2.GoodsStart != 1 && obj.Data2.GoodsStart != 3)
                    {
                        SqlSugarHelper.Do(db =>
                        {
                            var error = new BaseErrorinfoWcs()
                            {
                                ContBarCode = barcode,
                                Message = $"帘线异常排出口,颜色不匹配,识别器识别为:{color},自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                                Count = 1,
                                Memo = "WCS"
                            };
                            var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                            if (errorinfoWcs != null)
                            {
                                errorinfoWcs.Count = errorinfoWcs.Count++;
                                errorinfoWcs.AddTime = DateTime.Now;
                                db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                            }
                            else
                            {
                                db.Default.Insertable(error).ExecuteCommand();
                            }
                        });

                        World.Log($"帘线扫码{obj.Entity.Code}交互结束时间:工字轮条码{barcode} | 颜色标识 {obj.Data2.GoodsStart} " + DateTime.Now.ToString("yyyyMMddHHmmss"));
                        obj.Data.TaskNumber = obj.Data2.TaskNumber;
                        obj.Data.GoodsEnd = 10;
                        obj.Data.VoucherNo++;
                        return;
                    }
                }
                if (obj.Data2.GoodsStart == 0) //没有识别出颜色标识
                {
                    SqlSugarHelper.Do(db =>
                    {
                        var error = new BaseErrorinfoWcs()
                        {
                            ContBarCode = barcode,
                            Message = $"帘线异常排出口,没有识别出工字轮的颜色,521db块颜色值:{color},自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                            Count = 1,
                            Memo = "WCS"
                        };
                        var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                        if (errorinfoWcs != null)
                        {
                            errorinfoWcs.Count = errorinfoWcs.Count++;
                            errorinfoWcs.AddTime = DateTime.Now;
                            db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                        }
                        else
                        {
                            db.Default.Insertable(error).ExecuteCommand();
                        }
                    });
                    obj.Data.TaskNumber = obj.Data2.TaskNumber;
                    obj.Data.GoodsEnd = 10;
                    obj.Data.VoucherNo++;
                    return;
                }

            }


            if (!res.Success)
            {
                SqlSugarHelper.Do(db =>
                {
                    var error = new BaseErrorinfoWcs()
                    {
                        ContBarCode = barcode,
                        Message = $"帘线异常排出口,WMS分配楼层失败,原因：{res.ResMsg},{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                        Count = 1,
                        Memo = "WCS"
                    };
                    var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                    if (errorinfoWcs != null)
                    {
                        errorinfoWcs.Count = errorinfoWcs.Count++;
                        errorinfoWcs.AddTime = DateTime.Now;
                        db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                    }
                    else
                    {
                        db.Default.Insertable(error).ExecuteCommand();
                    }
                });

                World.Log(res.ResMsg);
                MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = "帘线扫码" + obj.Entity.Code + "，" + res.ResMsg, Time = DateTime.Now }));
                //throw new Exception(res.ResMsg);
                obj.Data.TaskNumber = obj.Data2.TaskNumber;
                obj.Data.GoodsEnd = 10;
                obj.Data.VoucherNo++;
                return;
            }
            if (res.Memo1 == "1" && obj.Data2.Mode != StationMode.Automatic) //1 -> bs60
            {
                SqlSugarHelper.Do(db =>
                {
                    var error = new BaseErrorinfoWcs()
                    {
                        ContBarCode = barcode,
                        Message = $"帘线异常排出口,当前条码规格为BS60小轮子,外检识别不匹配,外检值:{obj.Data2.Mode},自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                        Count = 1,
                        Memo = "WCS"
                    };
                    var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                    if (errorinfoWcs != null)
                    {
                        errorinfoWcs.Count = errorinfoWcs.Count++;
                        db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                    }
                    else
                    {
                        db.Default.Insertable(error).ExecuteCommand();
                    }
                });
                obj.Data.TaskNumber = obj.Data2.TaskNumber;
                obj.Data.GoodsEnd = 10;
                obj.Data.VoucherNo++;
                return;
            }
            if (res.Memo1 == "2" && obj.Data2.Mode != StationMode.Manual) //2 -> bs80
            {
                SqlSugarHelper.Do(db =>
                {
                    var error = new BaseErrorinfoWcs()
                    {
                        ContBarCode = barcode,
                        Message = $"帘线异常排出口,当前条码规格为BS80/33大轮子,外检识别不匹配,外检值:{obj.Data2.Mode},自动异常排出,{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}",
                        Count = 1,
                        Memo = "WCS"
                    };
                    var errorinfoWcs = db.Default.Queryable<BaseErrorinfoWcs>().With(SqlWith.NoLock).Where(v => v.ContBarCode == error.ContBarCode && v.Message == error.Message).First();
                    if (errorinfoWcs != null)
                    {
                        errorinfoWcs.Count = errorinfoWcs.Count++;
                        db.Default.Updateable(errorinfoWcs).ExecuteCommand();
                    }
                    else
                    {
                        db.Default.Insertable(error).ExecuteCommand();
                    }
                });
                obj.Data.TaskNumber = obj.Data2.TaskNumber;
                obj.Data.GoodsEnd = 10;
                obj.Data.VoucherNo++;
                return;
            }
            var task = res.ResData.TaskNo;
            if (task == 0)
            {
                MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = obj.Entity.Code + res.ResMsg, Time = DateTime.Now }));
                throw new Exception(res.ResMsg);
            }
            //var t1 = MainWorld.Redis.Get("Floor1Torsch");
            //var t2 = MainWorld.Redis.Get("Floor2Torsch");
            //var t3 = MainWorld.Redis.Get("Floor3Torsch");
            //var t11 = MainWorld.Redis.Get("Floor1Preel");
            //var t22 = MainWorld.Redis.Get("Floor2Preel");
            //var t33 = MainWorld.Redis.Get("Floor3Preel");
            //var t4 = MainWorld.Redis.Get("TaskCountNo");
            //if (int.Parse(t1) + int.Parse(t11) >= int.Parse(t4) && res.ResData.Floor == 1)
            //{
            //    MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = obj.Entity.Code + ",一楼环线出库已满，优先满足出库，暂停入库", Time = DateTime.Now }));
            //    throw new Exception("一楼环线出库已满，优先满足出库，暂停入库");
            //}
            //else if (int.Parse(t2) + int.Parse(t22) >= int.Parse(t4) && res.ResData.Floor == 2)
            //{
            //    MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = obj.Entity.Code + ",一楼环线出库已满，优先满足出库，暂停入库", Time = DateTime.Now }));
            //    throw new Exception("二楼环线出库已满，优先满足出库，暂停入库");
            //}
            //else if (int.Parse(t3) + int.Parse(t33) >= int.Parse(t4) && res.ResData.Floor == 3)
            //{
            //    MainWorld.Redis.Set($"Sx:WCSErrorInfo:{obj.Entity.Code}", JsonConvert.SerializeObject(new RedisError() { Equip = obj.Entity.Code, Con = obj.Entity.Code + ",一楼环线出库已满，优先满足出库，暂停入库", Time = DateTime.Now }));
            //    throw new Exception("三楼环线出库已满，优先满足出库，暂停入库");
            //}
            SqlSugarHelper.Do(db =>
            {
                var task = db.Default.Queryable<WCS_TaskInfo>().Where(v => v.ID == res.ResData.TaskNo).First();

                task.Floor = res.ResData.Floor;
                task.Status = Entity.TaskStatus.帘线扫码;
                task.EditTime = DateTime.Now;
                db.Default.Updateable(task).ExecuteCommand();

                task.AddWCS_TASK_DTL(db.Default, obj.Entity.Code, "帘线扫码");

                obj.Data.TaskNumber = res.ResData.TaskNo;
                obj.Data.GoodsEnd = res.ResData.Floor;

                obj.Data.VoucherNo++;
            });
            World.Log($"帘线扫码{obj.Entity.Code}交互结束时间:工字轮条码{barcode} | " + DateTime.Now.ToString("yyyyMMddHHmmss"));
        }

        public override bool Select(Device dev)
        {
            return dev.HasFlag(DeviceFlags.扫码 | DeviceFlags.帘线);//&&dev.Code== "2258";
        }
    }
}
