﻿using Newtonsoft.Json;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Castle.Core.Internal;
using ServiceCenter.SqlSugars;
using SqlSugar;
using WCS.Core;
using WCS.Entity.Protocol.BCR;
using WCS.Entity.Protocol.Station;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.WebApi.Models.WCS.Request;
using WCS.WorkEngineering.Worlds;
using TaskStatus = WCS.Entity.TaskStatus;

namespace WCS.WorkEngineering.Systems.装箱码垛
{
    //[BelongTo(typeof(PalletizingWorld))]
    [Description("进缓存记录")]
    public class 进缓存记录 : DeviceSystem<Device<IBCR83>>
    {
        protected override bool ParallelDo => true;

        public override void Do(Device<IBCR83> obj)
        {
            //缓存条码
            var cacheBcrList = obj.Data.GetBcrCodeList().Where(x => !x.IsNullOrEmpty());

            //循环处理所有缓存条码组中没有的条码
            foreach (var bcrCode in cacheBcrList)
            {
                try
                {
                    SqlSugarHelper.Do(db =>
                    {
                        var task = db.Default.Queryable<WCS_TaskInfo>().Where(v => bcrCode == v.BarCode && v.Status < TaskStatus.码垛缓存扫码).First();
                        if (task == null) return;
                        task.Status = Entity.TaskStatus.码垛缓存扫码;
                        task.EditTime = DateTime.Now;
                        db.Default.Updateable(task).ExecuteCommand();
                        task.AddWCS_TASK_DTL(db.Default, obj.Entity.Code, "码垛缓存");
                    });
                }
                catch (Exception e)
                {
                    World.Log($"{bcrCode}:----{e.Message}", LogLevelEnum.High);
                }
            }
        }

        public override bool Select(Device dev)
        {
            return dev.Code == "5041";
        }
    }
}
